@@ -18,14 +18,11 @@ from xpra.log import Logger
 log = Logger("gtk", "client")
 opengllog = Logger("gtk", "opengl")
 cursorlog = Logger("gtk", "client", "cursor")
-screenlog = Logger("gtk", "client", "screen")
 framelog = Logger("gtk", "client", "frame")
-menulog = Logger("gtk", "client", "menu")
 filelog = Logger("gtk", "client", "file")
 clipboardlog = Logger("gtk", "client", "clipboard")
 notifylog = Logger("gtk", "notify")
 grablog = Logger("client", "grab")
-authlog = Logger("client", "auth")
 
 from xpra.gtk_common.quit import (gtk_main_quit_really,
                            gtk_main_quit_on_fatal_exceptions_enable)
@@ -33,11 +30,8 @@ from xpra.util import updict, pver, iround, flatten_dict, envbool, typedict, rep
     DEFAULT_METADATA_SUPPORTED, XPRA_OPENGL_NOTIFICATION_ID
 from xpra.os_util import bytestostr, strtobytes, hexstr, WIN32, OSX, POSIX, PYTHON3
 from xpra.simple_stats import std_unit
-from xpra.net.compression import Compressible
-from xpra.net.file_transfer import FileTransferHandler
 from xpra.exit_codes import EXIT_PASSWORD_REQUIRED
 from xpra.scripts.config import TRUE_OPTIONS, FALSE_OPTIONS
-from xpra.client.window_border import WindowBorder
 from xpra.gtk_common.cursor_names import cursor_types
 from xpra.gtk_common.gtk_util import get_gtk_version_info, scaled_image, get_default_cursor, color_parse, gtk_main, \
             new_Cursor_for_display, new_Cursor_from_pixbuf, icon_theme_get_default, \
@@ -49,7 +43,7 @@ from xpra.gtk_common.gobject_util import no_arg_signal
 from xpra.client.ui_client_base import UIXpraClient
 from xpra.client.gobject_client_base import GObjectXpraClient
 from xpra.client.gtk_base.gtk_keyboard_helper import GTKKeyboardHelper
-from xpra.client.gtk_base.session_info import SessionInfo
+from xpra.client.mixins.window_manager import WindowClient
 from xpra.platform.paths import get_icon_filename
 from xpra.platform.gui import get_window_frame_sizes, get_window_frame_size, system_bell, get_fixed_cursor_size, get_menu_support_function, get_wm_name
 
@@ -59,13 +53,11 @@ METADATA_SUPPORTED = os.environ.get("XPRA_METADATA_SUPPORTED")
 USE_LOCAL_CURSORS = envbool("XPRA_USE_LOCAL_CURSORS", True)
 EXPORT_ICON_DATA = envbool("XPRA_EXPORT_ICON_DATA", True)
 SAVE_CURSORS = envbool("XPRA_SAVE_CURSORS", False)
+CLIPBOARD_NOTIFY = envbool("XPRA_CLIPBOARD_NOTIFY", True)
 
 
 class GTKXpraClient(GObjectXpraClient, UIXpraClient):
-    __gsignals__ = {
-        "first-ui-received"         : no_arg_signal,
-        "keyboard-sync-toggled"     : no_arg_signal,
-        }
+    __gsignals__ = {}
     #add signals from super classes (all no-arg signals)
     for signal_name in UIXpraClient.__signals__:
         __gsignals__[signal_name] = no_arg_signal
@@ -105,7 +97,10 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         self._ref_to_group_leader = {}
         self._group_leader_wids = {}
         self._set_window_menu = get_menu_support_function()
-        self.connect("scaling-changed", self.reset_windows_cursors)
+        try:
+            self.connect("scaling-changed", self.reset_windows_cursors)
+        except TypeError:
+            log("no 'scaling-changed' signal")
         #detect when the UI thread isn't responding:
         self.UI_watcher = None
         self.connect("first-ui-received", self.start_UI_watcher)
@@ -142,7 +137,9 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         return  self.exit_code
 
     def gtk_main(self):
+        log("GTKXpraClient.gtk_main() calling %s", gtk_main)
         gtk_main()
+        log("GTKXpraClient.gtk_main() ended")
 
 
     def quit(self, exit_code=0):
@@ -153,21 +150,24 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             #if for some reason cleanup() hangs, maybe this will fire...
             self.timeout_add(4*1000, self.exit)
             #try harder!:
-            def force_quit():
-                from xpra import os_util
-                os_util.force_quit()
-            self.timeout_add(5*1000, force_quit)
+            self.timeout_add(5*1000, self.force_quit)
         self.cleanup()
         log("GTKXpraClient.quit(%s) cleanup done, main_level=%s", exit_code, gtk.main_level())
         if gtk.main_level()>0:
             log("GTKXpraClient.quit(%s) main loop at level %s, calling gtk quit via timeout", exit_code, gtk.main_level())
             self.timeout_add(500, self.exit)
 
+    def force_quit(self):
+        from xpra.os_util import force_quit
+        log("GTKXpraClient.force_quit() calling %s", force_quit)
+        force_quit()
+
     def exit(self):
         log("GTKXpraClient.exit() calling %s", gtk_main_quit_really)
         gtk_main_quit_really()
 
     def cleanup(self):
+        log("GTKXpraClient.cleanup()")
         if self.session_info:
             self.session_info.destroy()
             self.session_info = None
@@ -206,7 +206,11 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         #subclasses may add their toolkit specific variants
         #by overriding this method
         #use the native ones first:
-        ncs = UIXpraClient.get_notifier_classes(self)
+        from xpra.client import mixin_features
+        assert mixin_features.notifications
+        from xpra.client.mixins.notifications import NotificationClient
+        assert isinstance(self, NotificationClient)
+        ncs = NotificationClient.get_notifier_classes(self)
         try:
             from xpra.gtk_common.gtk_notifier import GTK_Notifier
             ncs.append(GTK_Notifier)
@@ -318,6 +322,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             if size>=45:
                 log.warn("border size is too high: %s, clipping it", size)
                 size = 45
+        from xpra.client.window_border import WindowBorder
         self.border = WindowBorder(enabled, color.red/65536.0, color.green/65536.0, color.blue/65536.0, alpha, size)
         log("parse_border(%s)=%s", border_str, self.border)
 
@@ -389,6 +394,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
                 filelog.warn(" received data type=%s, url=%s, print=%s, open=%s", dtype, url, printit, openit)
                 return False
             return True
+        from xpra.net.file_transfer import FileTransferHandler
         return FileTransferHandler.accept_data(self, send_id, dtype, url, printit, openit)
 
     def file_size_warning(self, action, location, basefilename, filesize, limit):
@@ -487,6 +493,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         p = self._protocol
         if p:
             conn = p._conn
+        from xpra.client.gtk_base.session_info import SessionInfo
         self.session_info = SessionInfo(self, self.session_name, pixbuf, conn, self.get_pixbuf)
         self.session_info.set_args(*args)
         self.session_info.show_all()
@@ -578,12 +585,12 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
                     wm_name = None
                 try:
                     if len(v)==8:
-                        v = v[4:]
                         if first_time("invalid-frame-extents"):
                             framelog.warn("Warning: invalid frame extents value '%s'", v)
                             if wm_name:
                                 framelog.warn(" this is probably a bug in '%s'", wm_name)
-                            framelog.warn(" using '%s' instead", v)
+                            framelog.warn(" using '%s' instead", v[4:])
+                        v = v[4:]
                     l, r, t, b = v
                     wfs["frame"] = (l, r, t, b)
                     wfs["offset"] = (l, t)
@@ -605,10 +612,11 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         return tray_list
 
     def get_tray_classes(self):
-        return self._add_statusicon_tray(UIXpraClient.get_tray_classes(self))
+        from xpra.client.mixins.tray import TrayClient
+        return self._add_statusicon_tray(TrayClient.get_tray_classes(self))
 
     def get_system_tray_classes(self):
-        return self._add_statusicon_tray(UIXpraClient.get_system_tray_classes(self))
+        return self._add_statusicon_tray(WindowClient.get_system_tray_classes(self))
 
 
     def supports_system_tray(self):
@@ -701,7 +709,10 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
                 pass
         log("metadata.supported: %s", ms)
         capabilities["metadata.supported"] = ms
-        capabilities["pointer.grabs"] = True
+        updict(capabilities, "pointer", {
+            "grabs" : True,
+            "relative" : True,
+            })
         updict(capabilities, "window", {
                "initiate-moveresize"    : True,
                "configure.pointer"      : True,
@@ -851,8 +862,15 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
 
     def process_ui_capabilities(self):
         UIXpraClient.process_ui_capabilities(self)
-        if self.server_randr:
-            display = display_get_default()
+        #this requires the "DisplayClient" mixin:
+        if not hasattr(self, "screen_size_changed"):
+            return
+        display = display_get_default()
+        if is_gtk3():
+            #always one screen per display:
+            screen = gdk.Screen.get_default()
+            screen.connect("size-changed", self.screen_size_changed)
+        else:
             i=0
             while i<display.get_n_screens():
                 screen = display.get_screen(i)
@@ -871,6 +889,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         grablog("keyboard_grab(..)=%s", GRAB_STATUS_STRING.get(r, r))
 
     def window_ungrab(self):
+        grablog("window_ungrab()")
         gdk.pointer_ungrab(0)
         gdk.keyboard_ungrab(0)
 
@@ -937,9 +956,12 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             for x in str(e).split("\n"):
                 opengllog.error(" %s", x)
             self.opengl_props["info"] = str(e)
-            summary = "OpenGL Setup Failure"
+            summary = "Xpra OpenGL Acceleration Failure"
             body = str(e)
-            self.may_notify(XPRA_OPENGL_NOTIFICATION_ID, summary, body, icon_name="opengl")
+            def delayed_notify():
+                if self.exit_code is None:
+                    self.may_notify(XPRA_OPENGL_NOTIFICATION_ID, summary, body, icon_name="opengl")
+            self.timeout_add(2*1000, delayed_notify)
 
         if warnings:
             if enable_option in ("", "auto"):
@@ -984,7 +1006,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             self.GLClientWindowClass = gl_client_window_module.GLClientWindow
             self.client_supports_opengl = True
             #only enable opengl by default if force-enabled or if safe to do so:
-            self.opengl_enabled = (enable_option in (TRUE_OPTIONS+["auto"])) or self.opengl_props.get("safe", False)
+            self.opengl_enabled = (enable_option in (list(TRUE_OPTIONS)+["auto"])) or self.opengl_props.get("safe", False)
             self.gl_texture_size_limit = self.opengl_props.get("texture-size-limit", 16*1024)
             self.gl_max_viewport_dims = self.opengl_props.get("max-viewport-dims", (self.gl_texture_size_limit, self.gl_texture_size_limit))
             if min(self.gl_max_viewport_dims)<4*1024:
@@ -1049,6 +1071,9 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             driver_info = self.opengl_props.get("renderer") or self.opengl_props.get("vendor") or "unknown card"
             if self.opengl_enabled:
                 opengllog.info("OpenGL enabled with %s", driver_info)
+                #don't try to handle video dimensions bigger than this:
+                mvs = min(8192, self.gl_texture_size_limit)
+                self.video_max_size = (mvs, mvs)
             elif self.client_supports_opengl:
                 opengllog("OpenGL supported with %s, but not enabled", driver_info)
         except ImportError as e:
@@ -1153,7 +1178,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
 
     def destroy_window(self, wid, window):
         #override so we can cleanup the group-leader if needed,
-        UIXpraClient.destroy_window(self, wid, window)
+        WindowClient.destroy_window(self, wid, window)
         group_leader = window.group_leader
         if group_leader is None or len(self._group_leader_wids)==0:
             return
@@ -1254,6 +1279,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
                 clipboardlog("clipboard is disabled, not sending clipboard packet")
                 return
             #handle clipboard compression if needed:
+            from xpra.net.compression import Compressible
             packet = list(parts)
             for i in range(len(packet)):
                 v = packet[i]
@@ -1298,7 +1324,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         return helperClass(clipboard_send, clipboard_progress, **kwargs)
 
     def clipboard_notify(self, n):
-        if not self.tray:
+        if not self.tray or not CLIPBOARD_NOTIFY:
             return
         clipboardlog("clipboard_notify(%s)", n)
         if n>0 and self.clipboard_enabled:
